package gp_dbc.config;

import bspkrs.bspkrscore.fml.Reference;
import cpw.mods.fml.client.event.ConfigChangedEvent;
import cpw.mods.fml.common.eventhandler.SubscribeEvent;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

import gp_dbc.Main;
import gp_dbc.utils.Pos;
import net.minecraft.client.Minecraft;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.ChunkCoordinates;
import net.minecraftforge.common.config.Configuration;
import net.minecraftforge.common.config.Property;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import static net.minecraft.util.StatCollector.translateToFallback;

public class ConfigManager {

    private static Configuration config;
    public static boolean updateCheck = true;

    public static final String CATEGORY_GP = "Galactic Patrol";

    public static String DIR_SAVE = "./" + Main.MODID + "/";

    public static byte MIN_ALIGN_TO_JOIN = 33;
    public static byte MAX_ALIGN_TO_JOIN = 100;

    public static int FORMAT_TIME_JAIL = 4;

    public static int TIME_HANDCUFFING = 10; // seconds
    public static int TIME_TO_GO_IN_JAIL = 20; // seconds

    static public int SEC_TO_BE_IN_JAIL = 120; // second
    static public String POS_EXIT_JAIL = ""; //"-247 66 275 20";
    static public Pos pos_exit_jail;

    public static void init(String configDir) {
        if (config != null)
            return ;

        File path = new File(configDir + "/" + Main.MODID + ".cfg");

        config = new Configuration(path);
        loadConfiguration();
    }

    private static void loadConfiguration() {
        updateCheck = config.getBoolean("Check For Updates", Configuration.CATEGORY_GENERAL, true, "Allow this mod to check for updates.");

        List<String> order = new ArrayList<String>();

        DIR_SAVE = addVariableConfig(order, CATEGORY_GP, "DIR_SAVE",
                "Directory where data is saved. [default: " + "./" + Main.MODID + "/" + " ]", "./" + Main.MODID + "/").getString();

        MIN_ALIGN_TO_JOIN = (byte)addVariableConfig(order, CATEGORY_GP, "MIN_ALIGN_TO_JOIN",
                "Minimum alignment to join galactic patrol. [default: 33]", "33").getInt();

        MAX_ALIGN_TO_JOIN = (byte)addVariableConfig(order, CATEGORY_GP, "MAX_ALIGN_TO_JOIN",
                "Maximum alignment to join galactic patrol. [default: 100]", "100").getInt();

        POS_EXIT_JAIL = addVariableConfig(order, CATEGORY_GP, "POS_EXIT_JAIL",
                "Spawn point when a jailer leaves the prison. It's mandatory to put a player in jail. \"<x> <y> <z> <dimension id>\" [default: \"\"]", "").getString();
        try {
            String[] split = ConfigManager.POS_EXIT_JAIL.split(" ");
            pos_exit_jail = new Pos(Integer.parseInt(split[0]), Integer.parseInt(split[1]), Integer.parseInt(split[2]), Integer.parseInt(split[3]));
        } catch (Exception e) {
            System.out.println(e.getMessage());
            ChunkCoordinates spawnPoint = MinecraftServer.getServer().worldServerForDimension(0).getSpawnPoint();;
            pos_exit_jail = new Pos(spawnPoint.posX, spawnPoint.posY, spawnPoint.posZ, 0);
        }

        TIME_HANDCUFFING = (byte)addVariableConfig(order, CATEGORY_GP, "TIME_HANDCUFFING",
                "Time to handcuffing someone who is not K.O, in seconds. [default: 10]", "10").getInt();

        TIME_TO_GO_IN_JAIL = (byte)addVariableConfig(order, CATEGORY_GP, "TIME_TO_GO_IN_JAIL",
                "Time to go in jail when someone is handcuffed. [default: 20]", "20").getInt();

        config.setCategoryPropertyOrder(CATEGORY_GP, order);


        if (config.hasChanged())
            config.save();
    }
    
    private static Property addVariableConfig(List<String> order, String category, String key, String comment, String defaultValue) {
        Property prop;
        prop = config.get(category, key, defaultValue);
        prop.comment = comment;
        order.add(prop.getName());
        return prop;
    }

    @SubscribeEvent
    public void onConfigurationChangeEvent(ConfigChangedEvent.OnConfigChangedEvent event) {
        if (event.modID.equalsIgnoreCase(Main.MODID))
            loadConfiguration();
    }

    public static Configuration getConfiguration() {return config;}

}
